/* 	Attest.java
 *	Copyright (c) 2007-2009, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-feb-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.attest;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.MouseInfo;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.ResourceBundle;
import java.util.jar.JarFile;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;

import org.brains2b.attest.gui.AttestView;
import org.brains2b.attest.gui.JarView;
import org.brains2b.console.ArgumentException;
import org.brains2b.console.ArgumentMap;
import org.brains2b.console.Main;
import org.brains2b.log.Logger;

public class Attest extends Main implements ActionListener {

	private AttestView m_attest;
	private JarView m_jar;
	
	class TitleListener implements PropertyChangeListener {

		private JFrame m_frame;
		private String m_fixed;
		
		public TitleListener(JFrame frame, String fixed) {
			m_frame = frame;
			m_fixed = fixed;
		}
		
		public void propertyChange(PropertyChangeEvent evt) {
			if (!JarDescriptor.PROPERTY.equals(evt.getPropertyName())) return;
			if (evt.getOldValue()==null || !evt.getOldValue().equals(evt.getNewValue())) {
				ResourceBundle rs = ResourceBundle.getBundle("org.brains2b.attest.resource.Attest");
				JarDescriptor jar = (JarDescriptor) evt.getNewValue();
				m_frame.setTitle(MessageFormat.format(rs.getString("ApplTitle"), m_fixed,jar.getFile().getName()));
				((JTabbedPane) m_frame.getContentPane()).setEnabledAt(1, true);
			}
			
		}
		
	}
	
    public Attest() {
        super();
        init();
    }
    
    public static void main(String[] args) {
        Attest attest = new Attest();
        attest.makeArgumentList(args);
        //If we asked for console or there are no Graphics, print an quit
        if (GraphicsEnvironment.isHeadless() || "true".equals(attest.getArgument("console"))) {
           try {
        	  File f = new File(attest.getArgument("jar"));
        	  JarFile jar = new JarFile(f);
        	  jar.getManifest().write(System.out);
           } catch (Exception ex) {
        	   Logger.printException(ex);
           }
           System.exit(0);
         } else {
        	attest.doIt();
         }
        Logger.println(System.getProperty("os.name"),Logger.DEBUG);
        if ("Mac OS X".equals(System.getProperty("os.name")) && args.length==0) {
        		try {
	        		OsStub macOsx= (OsStub)  Class.forName("org.brains2b.attest.macosx.MacOsXHandler").newInstance();
	        		macOsx.addActionListener(attest);
        		} catch (Exception ex) {
        			attest.showException(ex);
        		}
        }
    }
    
    public void actionPerformed(ActionEvent e) {
    	String fileName = ((OsStub) e.getSource()).getFilename();
    	openFile(fileName);
    }
    
    private void openFile(String fileName) {
    	try {
    		File f = new File(fileName);
    		JarDescriptor jar = new JarDescriptor(f);
    		
    		
    		m_attest.setManifest(jar);
    	} catch (IOException iex) {
    		showException(iex);
    	}
    	
    }
    
    private void doIt() {
    	final Font f= UIManager.getFont("TextField.font");
        UIManager.put("Tree.font", f);
        UIManager.put("Label.font", f);
        UIManager.put("MenuItem.font", f);
        UIManager.put("Menu.font", f);
        UIManager.put("TabbedPane.font", f);
        ResourceBundle rs = ResourceBundle.getBundle("org.brains2b.attest.resource.Attest");
        final JFrame frm = new JFrame();
        frm.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    	frm.setResizable(false);
    	String title = this.getClass().getPackage().getSpecificationTitle() + " " + this.getClass().getPackage().getSpecificationVersion();
    	frm.setTitle(title);
        Dimension d= frm.getToolkit().getScreenSize();
        d.height=d.height*40/100;
        d.width=d.width*30/100;
        //JDK 1.5 dependant code
        frm.setLocation(MouseInfo.getPointerInfo().getLocation());
        //
        frm.setSize(d);
        
        JTabbedPane pane = new JTabbedPane();
        frm.setContentPane(pane);
        m_attest =new AttestView();
        pane.addTab(rs.getString("TabManifest"), m_attest);
        
        m_jar = new JarView();
        pane.addTab(rs.getString("TabContent"), m_jar);
        pane.setEnabledAt(1, false);
        
        m_attest.addPropertyChangeListener(JarDescriptor.PROPERTY, m_jar);
        m_attest.addPropertyChangeListener(new TitleListener(frm, title));
        ActionMap map = new ActionMap();
        map.put("frm.open", new AbstractAction(rs.getString("MnuOpen")) {
        	JFileChooser  jfc= new JFileChooser();
        	public void actionPerformed(ActionEvent e) {
        		jfc.setFileFilter(new FileFilter() {
        			@Override
        			public boolean accept(File f) {
        				if (f.isDirectory()) return true;
        				final String s = f.getName().toLowerCase();
        				return (s.endsWith(".jar") || s.endsWith(".war") || s.endsWith(".ear"));
        			}
        			
        			@Override
        			public String getDescription() {
        				return "*.jar; *.war; *.ear";
        			}
        		});
        		jfc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        		if (JFileChooser.APPROVE_OPTION ==jfc.showOpenDialog(frm)) {
        			openFile(jfc.getSelectedFile().getAbsolutePath());
        		}
        	}
        	
        	@Override
        	public Object getValue(String key) {
        		if (ACCELERATOR_KEY.equals(key)) {
        			return KeyStroke.getKeyStroke('O',KeyEvent.CTRL_DOWN_MASK);
        		} else if (MNEMONIC_KEY.equals(key)) {
        			return Integer.valueOf('O');
        		}
        		return super.getValue(key);
        	}
        	
        });
        
        map.put("frm.close", new AbstractAction(rs.getString("MnuClose")) {
        	
        	public void actionPerformed(ActionEvent e) {
        		frm.dispose();
        	}
        	
        	@Override
        	public Object getValue(String key) {
        		if (ACCELERATOR_KEY.equals(key)) {
        			return KeyStroke.getKeyStroke(KeyEvent.VK_F4,KeyEvent.ALT_DOWN_MASK);
        		} else if (MNEMONIC_KEY.equals(key)) {
        			return Integer.valueOf('C');
        		}
        		return super.getValue(key);
        	}
        });
        
        JMenuBar bar = new JMenuBar();
        JMenu menu = new JMenu(rs.getString("MnuFile"));
        menu.setMnemonic('f');
        bar.add(menu);
        menu.add(new JMenuItem(map.get("frm.open")));
        menu.addSeparator();
        menu.add(new JMenuItem(map.get("frm.close")));
        frm.setJMenuBar(bar);
        frm.setVisible(true);
        if (getArgument("jar")!=null) {
        	openFile(getArgument("jar"));
        }
    }
    
    private void init() { 
        ArgumentMap map =new ArgumentMap();
        map.addArgument("jar", false);
        map.addOption("console",'c',false);
        setArgumentMap(map);
    }
    
    protected void handleException(ArgumentException ex) {
        if (GraphicsEnvironment.isHeadless() || "true".equals(getArgument("console")) ) {
            super.handleException(ex);
        } else {
           showException(ex);
           System.exit(1);
        }
    }
    
    private void showException(Exception ex) {
    	if (GraphicsEnvironment.isHeadless() || "true".equals(getArgument("console"))) return;
    	ResourceBundle rs = ResourceBundle.getBundle("org.brains2b.attest.resource.Attest");
        JOptionPane.showMessageDialog(null, ex.getMessage(),rs.getString("ErrorTitle"),JOptionPane.ERROR_MESSAGE);
    }
    
}
