/* 	AttestView.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-feb-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.attest.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.ResourceBundle;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.text.BadLocationException;
import javax.swing.text.Element;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import org.brains2b.attest.JarDescriptor;

public class AttestView extends JPanel implements ActionListener,MouseMotionListener, MouseListener {
    
    private JTextPane m_textPane;
    private HashMap<String,JarDescriptor> m_children;
    private Stack<JarDescriptor> m_last;
    private JButton m_back;
    
    public AttestView() {
        super();
        init();
        m_last = new Stack();
    }
    
    private void init() {
    	ResourceBundle rs = ResourceBundle.getBundle("org.brains2b.attest.resource.Attest");
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        
        m_textPane = new JTextPane();
        m_textPane.setEditable(false);
        m_textPane.addMouseMotionListener(this);
        m_textPane.addMouseListener(this);
        add(new JScrollPane(m_textPane),BorderLayout.CENTER);
        
        StyledDocument doc = m_textPane.getStyledDocument();
        Style dflt= StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
        Style notfound=doc.addStyle("notfound", dflt);
        StyleConstants.setForeground(notfound, Color.red);
        StyleConstants.setStrikeThrough(notfound, true);
            
        Style link= doc.addStyle("link", doc.getStyle(StyleContext.DEFAULT_STYLE));
        StyleConstants.setForeground(link, Color.blue);
        StyleConstants.setUnderline(link, true);
        
        m_back= new JButton();
        m_back.setActionCommand("Back");
        m_back.addActionListener(this);
        m_back.setVisible(false);
        
        JButton b= new JButton(rs.getString("BtnClose"));
        b.setActionCommand("Close");
        b.addActionListener(this);
        JPanel p= new JPanel(new BorderLayout());
        p.setBorder(BorderFactory.createEmptyBorder(5, 0, 0, 0));
        p.add(m_back,BorderLayout.WEST);
        p.add(b,BorderLayout.EAST);
        add(p,BorderLayout.SOUTH);    
    }
    
    public void setManifest(JarDescriptor jar) {
    	firePropertyChange(JarDescriptor.PROPERTY, m_last.isEmpty()?null:m_last.peek(), jar);
        m_children= new HashMap();
        if (m_last.isEmpty() || m_last.peek()!=jar) {
        	m_last.push(jar);
        }
        ByteArrayOutputStream baos= new ByteArrayOutputStream((int) jar.getFile().length());
        String text=null;
        try {
            jar.getManifest().write(baos);
            baos.flush();
            text=new String(baos.toByteArray(),"ISO8859_1");
            text= text.replaceAll("\r\n ", ""); //Clean it up a bit
            text= text.replaceAll("\r\n", "\n");
            m_textPane.setText(text);
            
        } catch (Exception ex) {
            showException(ex);
            return;
        } finally {
            try {
                baos.close();
            } catch (IOException iex) {
            }
        }
        
        StyledDocument doc = m_textPane.getStyledDocument();
            
        doc.setCharacterAttributes(0, doc.getLength(), doc.getStyle(StyleContext.DEFAULT_STYLE), true);
        m_textPane.setCaretPosition(0);
        try {
            String as = (String) jar.getManifest().getMainAttributes().get(Attributes.Name.CLASS_PATH);
            if (as!=null) {
                int cidx= text.indexOf("Class-Path");
                StringTokenizer st= new StringTokenizer(as);
                while(st.hasMoreTokens()) {
                    String jarName = st.nextToken();
                    File fj =  new File(jar.getFile().getParent(),jarName);
                    
                    int idx= text.indexOf(jarName,cidx);
                    if (fj.exists()) {
                        jar = new JarDescriptor(fj);
                        Manifest jMf= jar.getManifest();
                        if (jMf!=null) {
                            doc.setCharacterAttributes(idx, jarName.length(), doc.getStyle("link"), false);
                            m_children.put(jarName, jar);
                        }
                    } else {
                        doc.setCharacterAttributes(idx, jarName.length(), doc.getStyle("notfound"), false);
                    }
                }
            }
        } catch(Exception blex) {
            showException(blex);
            return;
        }
        
        
    }
    
    public void actionPerformed(ActionEvent e) {
        if ("Close".equals(e.getActionCommand())) {
            SwingUtilities.getWindowAncestor(this).dispose();
        } else if ("Back".equals(e.getActionCommand())) {
        	m_last.pop();
            setManifest(m_last.peek());
            if (m_last.size()<=1) {
                m_back.setVisible(false);
            }
        }
    }
    
    public void mouseMoved(MouseEvent e) {
        int pos= m_textPane.viewToModel(e.getPoint());
        Element el= m_textPane.getStyledDocument().getCharacterElement(pos);
        if (el.getAttributes().containsAttribute(SimpleAttributeSet.NameAttribute,"link")) {
            m_textPane.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        } else {
            m_textPane.setCursor(Cursor.getDefaultCursor());
        }
    }
    
    public void mouseClicked(MouseEvent e) {
        int pos= m_textPane.viewToModel(e.getPoint());
        Element el= m_textPane.getStyledDocument().getCharacterElement(pos);
        if (el.getAttributes().containsAttribute(SimpleAttributeSet.NameAttribute,"link")) {
            try {
                String s=m_textPane.getStyledDocument().getText(el.getStartOffset(), el.getEndOffset()-el.getStartOffset());
                JarDescriptor jar= (JarDescriptor) m_children.get(s);
                if (jar!=null) {
                    setManifest(jar);
                    m_back.setText("< " + m_last.get(m_last.size()-2).getFile().getName());
                    m_back.setVisible(true);
                }
                    
            } catch (BadLocationException blex) {
                showException(blex);
            }
        }
    }
    
    private void showException(Exception ex) {
    	ResourceBundle rs = ResourceBundle.getBundle("org.brains2b.attest.resource.Attest");
        JOptionPane.showMessageDialog(this, ex.getMessage(),rs.getString("ErrorTitle"),JOptionPane.ERROR_MESSAGE);
    }
    
    public void mouseDragged(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }
 
    
}
