/* 	JarView.java
 *	Copyright (c) 2008-2009, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Sep 12, 2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.attest.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.ResourceBundle;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.brains2b.attest.JarDescriptor;
import org.brains2b.task.DefaultTask;
import org.brains2b.util.StringHelper;

public class JarView extends JPanel implements PropertyChangeListener {

	 class JarCellRenderer implements TreeCellRenderer {

		public Component getTreeCellRendererComponent(JTree arg0, Object arg1,
				boolean arg2, boolean arg3, boolean arg4, int arg5, boolean arg6) {
			
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) arg1;
			Object value = node.getUserObject();
			
			JLabel l = new JLabel();
			if (value instanceof JarFile) {
				l.setText(((JarFile) value).getName());
			} else if (value instanceof JarEntry) {
				JarEntry entry = (JarEntry) value;
				String name = entry.getName();
				
				if (!entry.isDirectory()) {
					name = name.substring(name.lastIndexOf('/')+1);
				} else {
					name = name.substring(0,name.length()-1).replace('/', '.');
				}
				if (arg2) {
					l.setOpaque(true);
					l.setBackground(UIManager.getColor("Tree.selectionBackground"));
				} else {
					l.setOpaque(false);
				}
				l.setText(name);
				
			}
			return l;
		}
		 
	 }
	 class InstantFind implements CaretListener {

		public void caretUpdate(CaretEvent arg0) {
			JTextField f= (JTextField) arg0.getSource();
			m_search.update(f.getText());
			
			if (m_search.isFinished()) {
				SwingUtilities.invokeLater(m_search);
			}
			
		}
		
	}
	
	 class Search extends DefaultTask {
		private String m_find;
		private DefaultMutableTreeNode m_root;
		public Search() {
			setFinished(true);
		}
		
		public void update(String find) {
			m_find = find;
		}
		 
		public void run() {
			setFinished(false);
			m_root = clone(m_fullRoot);
			m_model.setRoot(m_root);
			int result =0;
			if (!StringHelper.isEmpty(m_find)) {
				result = find (m_root,m_find);
			}
			
			m_model.reload();
			if (result>0) {
				TreePath path = new TreePath(m_model.getRoot());
				expandPath(path);
			}
			setFinished(true);
		}
		
		private int find(DefaultMutableTreeNode node, String find ) {
			if (!find.equals(m_find)) { //key was updated, start over
				m_root = clone(m_fullRoot);
				find (m_root,m_find);
				return Integer.MIN_VALUE;
			} 
			int result =0;
			while (node.getChildCount()>result) {
				result += find((DefaultMutableTreeNode) node.getChildAt(result),find)>0?1:0;
			}
			if (result>0) {
				return result;
			}
			String name = null;
			if (node.getUserObject() instanceof JarFile) {
				name = ((JarFile) node.getUserObject()).getName();
			} else {
				name = ((JarEntry) node.getUserObject()).getName();
				name = name.replace('/', '.');
			}
			if (result==0 && name.indexOf(find)==-1) {
				node.removeFromParent();
			} else {
				result=1;
			}
			return result;
		}
		
		private final DefaultMutableTreeNode clone(Object o) {
			DefaultMutableTreeNode root = (DefaultMutableTreeNode) o;
			DefaultMutableTreeNode clone =(DefaultMutableTreeNode) root.clone();
			deepClone(root,clone);
			return clone;
		}
		
		private final void deepClone(DefaultMutableTreeNode source, DefaultMutableTreeNode target) {
			for (int i=0;i<source.getChildCount();i++) {
				DefaultMutableTreeNode clone = (DefaultMutableTreeNode) ((DefaultMutableTreeNode) source.getChildAt(i)).clone();
				deepClone((DefaultMutableTreeNode) source.getChildAt(i),clone);
				target.add(clone);
			}
		}
		
		private void expandPath(TreePath path) {
			TreeNode node = (TreeNode) path.getLastPathComponent();
			if (node.getChildCount()>0) {
				m_tree.expandPath(path);
				TreeNode[] np = new TreeNode[path.getPathCount()+1];
    			System.arraycopy(path.getPath(), 0, np, 0, path.getPathCount());
				for (int i=0;i<node.getChildCount();i++) {
    				np[path.getPathCount()] = node.getChildAt(i);
    				expandPath(new TreePath(np));
				}
			}
		}
		
	 }
	 
	 class ClassTreeModel extends DefaultTreeModel {
		
		public ClassTreeModel() {
			super(null);
		}
		
	}
	
	private ClassTreeModel m_model;
	private DefaultMutableTreeNode m_fullRoot;
	private Search m_search = new Search();
	private JTree m_tree;
	private JTextField m_findField;
	
	public JarView() {
		super(); 
		init();
	}
	
	private void init() {
		ResourceBundle rs = ResourceBundle.getBundle("org.brains2b.attest.resource.Attest");
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		JPanel header = new JPanel(new BorderLayout(5,0));
		header.setBorder(BorderFactory.createEmptyBorder(5, 0, 5, 0));
		header.add(new javax.swing.JLabel(rs.getString("FindLabel")),BorderLayout.WEST);
		m_findField = new JTextField();
		m_findField.setToolTipText(rs.getString("FindTooltip"));
		m_findField.addCaretListener(new InstantFind());
		header.add(m_findField,BorderLayout.CENTER);
		add(header,BorderLayout.NORTH);
		
		m_model = new ClassTreeModel();
		m_tree = new JTree(m_model);
		m_tree.setCellRenderer(new JarCellRenderer());
		m_tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		add(new JScrollPane(m_tree),BorderLayout.CENTER);
	}
	
	public void setJar(JarDescriptor f) {
		
		m_findField.setText(null);
		
		DefaultMutableTreeNode node = new DefaultMutableTreeNode(f);
		Enumeration e = f.entries();
		HashMap m = new HashMap();
		while (e.hasMoreElements()) {
			JarEntry jar = (JarEntry) e.nextElement();
			if (jar.isDirectory()) {
				DefaultMutableTreeNode dir = (DefaultMutableTreeNode) m.get(jar.getName());
				if (dir==null) {
					dir = new DefaultMutableTreeNode(jar);
					m.put(jar.getName(), dir);
				} else {
					dir.setUserObject(jar);
				}
			} else {
				String path = jar.getName();
				int idx = path.lastIndexOf("/");
				if (idx>0) {
					path = path.substring(0,idx+1);
					DefaultMutableTreeNode dir = (DefaultMutableTreeNode) m.get(path);
					if (dir==null) {
						dir = new DefaultMutableTreeNode(new JarEntry(path));
						m.put(path, dir);
					}
					dir.add(new DefaultMutableTreeNode(jar));
					if (node.getIndex(dir)==-1) node.add(dir);
				}
			}
		}
		m_model.setRoot(node);
		m_model.reload();
		m_fullRoot = node;
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if (!JarDescriptor.PROPERTY.equals(evt.getPropertyName())) return;
		if (evt.getOldValue()==null || !evt.getOldValue().equals(evt.getNewValue())) {
			setJar((JarDescriptor) evt.getNewValue());
		}
		
	}
}
